package ga;

import ga.chromosomes.Chromosome;
import ga.problems.Problem;

import java.util.Random;

public class Algorithm {
    private Problem problemBeingSolved;
    private Population population;
    private static Random random = new Random();

    public Algorithm(Problem problem, int populationSize) {
        this.problemBeingSolved = problem;
        this.population = new Population(populationSize, problemBeingSolved);
        for (int i = 0; i < populationSize; i++)
            population.getIndividual(i).setScore(
                problemBeingSolved.evaluate(population.getIndividual(i)));
        population.computeStats();
    }

    /**
     * tournamentRounds == 0 means a one-round "tournament"; i.e. don't recurse
     * at all, and only do one comparison between two random chromosomes.
     * Setting tournamentRounds == x means to perform x recursion steps;
     * effectively, this means a tournament between 2^(x+1) randomly chosen
     * chromosomes.
     */
    private int selectChromosomeIndexByBinaryTournament(int tournamentRounds) {
        int index1, index2;
        if (tournamentRounds == 0) {
            index1 = getRandomChromosomeIndex();
            do {
                index2 = getRandomChromosomeIndex();
            } while (index1 == index2);
        }
        else {
            index1 = selectChromosomeIndexByBinaryTournament(tournamentRounds - 1);
            do {
                index2 = selectChromosomeIndexByBinaryTournament(tournamentRounds - 1);
            } while (index1 == index2);
        }
        if (problemBeingSolved.isScore1BetterThanScore2(
            population.getIndividual(index1).getScore(),
            population.getIndividual(index2).getScore()) > 0) {
            return index1;
        }
        return index2;
    }

    private int getRandomChromosomeIndex() {
        return (int) (random.nextDouble() * population.getPopulationSize());
    }

    public void goOneStep() {
        goOneStep(0);
    }

    public void goOneStep(int binaryTournamentRounds) {
        int tournamentWinner1 = selectChromosomeIndexByBinaryTournament(binaryTournamentRounds);
        int tournamentWinner2 = selectChromosomeIndexByBinaryTournament(binaryTournamentRounds);
        Chromosome newChromosome;
        if (tournamentWinner1 == tournamentWinner2) {
            newChromosome = population.getIndividual(tournamentWinner1).getChromosome().deepCopy();
        }
        else {
            newChromosome = problemBeingSolved.singlePointCrossover(
                population.getIndividual(tournamentWinner1).getChromosome(),
                population.getIndividual(tournamentWinner2).getChromosome());
        }
        Individual newIndividual = new Individual(newChromosome);
        problemBeingSolved.mutate(newIndividual.getChromosome());
        problemBeingSolved.evaluate(newIndividual);
        population.setIndividual(population.getWorstPosition(), newIndividual);
    }

    public Individual getBestKnownIndividual() {
        return population.getIndividual(population.getBestPosition());
    }

    public double getBestScore() {
        return population.getBestScore();
    }
}
