package ga;

import ga.chromosomes.Chromosome;
import ga.problems.Problem;

public class Population {
    private int populationSize;
    private Individual population[];
    private int bestPosition; // The index of the best individual
    private int worstPosition;// The index of the worst individual
    private double bestScore; // of the present population
    private double averageScore; // of the present population
    private double worstScore;// of the present population
    private double aggregateBestScore; // best fitness score ever found in
                                       // search
    private Problem problemBeingSolved;

    public Population(int inPopulationSize, Problem problemBeingSolved) {
        populationSize = inPopulationSize;
        population = new Individual[populationSize];
        this.problemBeingSolved = problemBeingSolved;
        for (int i = 0; i < populationSize; i++) {
            // get the template chromosome from the Problem class, because it is
            // the highest level of customization (i.e. a Problem may place
            // further constraints on Chromosome initialization than a
            // ChromosomeStrategy, which places further constraints on the basic
            // Chromosome constructor).
            Chromosome c = problemBeingSolved.getTemplateChromosome();
            population[i] = new Individual(c);
        }
        bestPosition = 0;
        worstPosition = 0;
        bestScore = 0.0;
        averageScore = 0.0;
        worstScore = Double.MAX_VALUE;
        aggregateBestScore = 0.0;
    }

    public int getPopulationSize() {
        return populationSize;
    }

    public Individual getIndividual(int index) {
        return population[index];
    }

    public void setIndividual(int index, Individual individual) {
        population[index] = individual;
        computeStats();
    }

    public void computeStats() {
        double individualScore, totalScore;

        totalScore = 0.0;
        worstScore = population[0].getScore();
        worstPosition = 0;
        bestScore = population[0].getScore();
        bestPosition = 0;
        for (int i = 0; i < populationSize; i++) {
            individualScore = population[i].getScore();
            if (problemBeingSolved.isScore1BetterThanScore2(individualScore, worstScore) < 0) {
                worstScore = individualScore;
                worstPosition = i;
            }
            if (problemBeingSolved.isScore1BetterThanScore2(individualScore, bestScore) > 0) {
                bestScore = individualScore;
                bestPosition = i;
            }
            if (problemBeingSolved.isScore1BetterThanScore2(individualScore, aggregateBestScore) > 0) {
                aggregateBestScore = individualScore;
            }
            totalScore += individualScore;
        }
        averageScore = totalScore / (double) populationSize;
    }

    public int getWorstPosition() {
        return worstPosition;
    }

    public int getBestPosition() {
        return bestPosition;
    }

    public double getBestScore() {
        return bestScore;
    }

    public void printStats() {
        System.out.print(aggregateBestScore);
        System.out.print("   ");
        System.out.print(bestScore);
        System.out.print("   ");
        System.out.print(averageScore);
        System.out.print("   ");
        System.out.print(worstScore);
        System.out.print("   ");
        System.out.print(bestPosition);
        System.out.print("   ");
        System.out.println(worstPosition);
    }
}
