package ga.problems;

import ga.Algorithm;
import ga.Individual;
import ga.chromosomes.Chromosome;
import ga.problems.helpers.ChromosomeStrategy;

import java.util.Random;

public abstract class Problem {
    protected int allelesPerChromosome = 1;
    protected ChromosomeStrategy chromosomeStrategy;

    protected static Random random = new Random();

    public Problem() {
        // defaults are set above
    }

    public Problem(int allelesPerChromosome) {
        this.allelesPerChromosome = allelesPerChromosome;
    }


    public void setMutationStrategy(ChromosomeStrategy chromosomeStrategy) {
        this.chromosomeStrategy = chromosomeStrategy;
    }
    
    public abstract double evaluate(Individual individual);

    public boolean resultHasBeenFound(Algorithm geneticAlgorithm, double targetScore) {
        return geneticAlgorithm.getBestKnownIndividual().getScore() >= targetScore;
    }
    
    public Chromosome singlePointCrossover(Chromosome chromosome1, Chromosome chromosome2){
        return chromosomeStrategy.singlePointCrossover(chromosome1, chromosome2);
    }

    public void mutate(Chromosome chromosome) {
        chromosomeStrategy.mutate(chromosome);
    }

    public Chromosome getTemplateChromosome() {
        return chromosomeStrategy.getTemplateChromosome();
    }

    public void print() {
        chromosomeStrategy.print();
    }

    /**
     * returns -1 if score1 is WORSE than score2
     * returns 1 if score1 is BETTER than score2
     * returns 0 if they are the same
     * The point of this method is that it leaves interpretation of what
     * "Better" and "worse" scores are, in the context of the given problem.
     */
    public int isScore1BetterThanScore2(double score1, double score2) {
        return compareScoresForMaximization(score1, score2);
    }

    protected int compareScoresForMaximization(double score1, double score2) {
        if (score1 == score2) return 0;
        if (score1 < score2) return -1;
        return 1;
    }

    protected int compareScoresForMinimization(double score1, double score2) {
        if (score1 == score2) return 0;
        if (score1 > score2) return -1;
        return 1;
    }
}
