package ga.problems;

import ga.Individual;
import ga.chromosomes.ByteChromosome;
import ga.problems.helpers.BasicChromosomeStrategy;

public class ProblemPPeaks extends Problem {
    private int lengthOfChromosome;
    private int numberOfPeaks = 1000;
    private short peaks[][];

    public ProblemPPeaks(int allelesPerChromosome, double crossoverProbability,
            double mutationsPerChromosome) {
        super(allelesPerChromosome);

        this.allelesPerChromosome = allelesPerChromosome;
        peaks = new short[numberOfPeaks][lengthOfChromosome];
        for (int p = 0; p < numberOfPeaks; p++) {
            for (int j = 0; j < lengthOfChromosome; j++)
                peaks[p][j] = (short) (random.nextBoolean() ? 1 : 0);
        }

        chromosomeStrategy = new BasicChromosomeStrategy(
            allelesPerChromosome,
            crossoverProbability,
            mutationsPerChromosome,
            this);
    }

    public double evaluate(Individual individual) {
        double score = 0.0; // Fitness of the individual
        int nearestPeak; // Bits in common with nearest peak
        int j, p, hammingDistance;

        for (nearestPeak = 0, p = 0; p < numberOfPeaks; p++) {
            // Compute Hamming distance
            for (hammingDistance = 0, j = 0; j < lengthOfChromosome; j++) {
                if (peaks[p][j] != ((ByteChromosome) individual.getChromosome()).getAllele(j)) {
                    hammingDistance++;
                }
            }

            if ((lengthOfChromosome - hammingDistance) > nearestPeak) {
                nearestPeak = lengthOfChromosome - hammingDistance;
            }
        }

        score = (double) nearestPeak / (double) lengthOfChromosome;
        individual.setScore(score);
        return score;
    }

}
