package ga.problems;

import static org.junit.Assert.*;
import ga.Individual;
import ga.chromosomes.IntChromosome;

import org.junit.Before;
import org.junit.Test;

public class ProblemTSPTest {
    private ProblemTSP problem;
    private Individual individual;

    @Before
    public void setUp() throws Exception {
        problem = new ProblemTSP(0.5, 1.0, "testValues.txt");
        int[] alleles = new int[]{0,1,2,3,4};
        IntChromosome c = new IntChromosome(alleles);
        individual = new Individual(c);
    }

    @Test
    public void testEvaluate() {
        assertEquals(20.0, problem.evaluate(individual), 0.01);
    }
    @Test
    public void testGetTemplateChromosome() {
        // do this test 10 times because it has randomness inside it.
        for (int i = 0; i < 10; i++) {
            IntChromosome c = (IntChromosome) problem.getTemplateChromosome(); //randomness within
            // make sure that the zero cityID was swapped to the start position
            assertEquals(0, c.getAllele(0));
            // make sure all other values are present (not overwritten)
            assertTrue(c.getAlleleIndexByValue(1)> 0);
            assertTrue(c.getAlleleIndexByValue(2)> 0);
            assertTrue(c.getAlleleIndexByValue(3)> 0);
            assertTrue(c.getAlleleIndexByValue(4)> 0);
        }
    }

    @Test
    public void testCompareScores() {
        // make sure the scoring mechanism is MINIMIZING
        assertTrue(problem.isScore1BetterThanScore2(0.0, 1.0) == 1);
        assertTrue(problem.isScore1BetterThanScore2(0.0, 0.0) == 0);
        assertTrue(problem.isScore1BetterThanScore2(1.0, 0.0) == -1);
    }
}
