package ga.problems.helpers;

import ga.chromosomes.ByteChromosome;
import ga.chromosomes.Chromosome;
import ga.problems.Problem;

public class BasicChromosomeStrategy extends ChromosomeStrategy {

    public BasicChromosomeStrategy(int allelesPerChromosome, double crossoverProbability,
            double mutationsPerChromosome, Problem problemBeingSolved) {
        super(
            allelesPerChromosome,
            crossoverProbability,
            mutationsPerChromosome,
            problemBeingSolved);
    }

    /**
     * Create a new individual and return it. Individual is created either as a
     * crossover of the two given individuals, or if, by crossoverProbability,
     * no crossover happens, then one of the two individuals is returned at
     * random.
     * 
     * This function will always return a new copy of an Individual object, not
     * referenced anywhere else.
     */
    public Chromosome singlePointCrossover(Chromosome chromosome1, Chromosome chromosome2) {
        Chromosome result = getTemplateChromosome();

        int crossoverPoint = (int) (random.nextDouble() * allelesPerChromosome);
        if (random.nextDouble() > crossoverProbability) {
            // If no crossover then randomly return one parent
            return random.nextBoolean() ? chromosome1.deepCopy() : chromosome2.deepCopy();
        }
        result.partiallyCopyFromOtherChromosome(chromosome1, 0, crossoverPoint - 1);
        result.partiallyCopyFromOtherChromosome(
            chromosome2,
            crossoverPoint,
            allelesPerChromosome - 1);
        return result;
    }

    public void mutate(Chromosome chromosome) {
        for (int i = 0; i < chromosome.getLength(); i++)
            if (random.nextDouble() < mutationProbability) {
                if (((ByteChromosome) chromosome).getAllele(i) == 1)
                    ((ByteChromosome) chromosome).setAllele(i, (byte) 0);
                else
                    ((ByteChromosome) chromosome).setAllele(i, (byte) 1);
            }
    }

    @Override
    public Chromosome getTemplateChromosome() {
        return new ByteChromosome(allelesPerChromosome);
    }
}
