package ga.problems.helpers;

import ga.chromosomes.Chromosome;
import ga.problems.Problem;

import java.util.Random;

public abstract class ChromosomeStrategy {
    protected static final Random random = new Random();
    protected int allelesPerChromosome;
    protected double mutationProbability;
    protected double mutationsPerChromosome;
    protected double crossoverProbability;
    protected Problem problemBeingSolved;

    public ChromosomeStrategy(int allelesPerChromosome, double crossoverProbability,
            double mutationsPerChromosome, Problem problemBeingSolved) {
        this.allelesPerChromosome = allelesPerChromosome;
        this.crossoverProbability = crossoverProbability;
        this.mutationsPerChromosome = mutationsPerChromosome;
        this.mutationProbability = mutationsPerChromosome / allelesPerChromosome;
        this.problemBeingSolved = problemBeingSolved;
    }

    public abstract Chromosome getTemplateChromosome();

    public abstract void mutate(Chromosome chromosome);

    public abstract Chromosome singlePointCrossover(Chromosome chromosome1, Chromosome chromosome2);

    public void print() {
        // System.out.printf("Chromosome Strategy: %s\n",
        // this.getClass().getName());
        // System.out.printf("Alleles per Chromosome: %d\n",
        // allelesPerChromosome);
        // System.out.printf("Mutation Probability: %f\n",
        // mutationsPerChromosome);
        // System.out.printf("Crossover Probability: %f\n",
        // crossoverProbability);
        System.out.printf(
            "%d\t%f\t%f\t",
            allelesPerChromosome,
            mutationsPerChromosome,
            crossoverProbability);
    }
}