package ga.problems.helpers;

import static org.junit.Assert.*;
import ga.chromosomes.IntChromosome;

import org.junit.Before;
import org.junit.Test;

public class UniqueIntegersChromosomeStrategyTest {

    private static final int ALLELES_PER_CHROMOSOME = 5;
    private UniqueIntegersChromosomeStrategy strategy;

    @Before
    public void setUp() throws Exception {
        strategy = new UniqueIntegersChromosomeStrategy(ALLELES_PER_CHROMOSOME, 0.5, 1.0, null);
    }

    @Test
    public void testGetTemplateChromosome() {
        for (int i = 0; i < 100; i++) {
            IntChromosome chromosome = (IntChromosome) strategy.getTemplateChromosome();
            asertLengthAndUniqueness(chromosome);
        }
    }

    private void asertLengthAndUniqueness(IntChromosome chromosome) {
        assertEquals(ALLELES_PER_CHROMOSOME, chromosome.getLength());
        IntegerChecklist checklist = new IntegerChecklist(ALLELES_PER_CHROMOSOME);
        for (int j = 0; j < ALLELES_PER_CHROMOSOME; j++) {
            assertTrue(checklist.checkOff(chromosome.getAllele(j)));
        }
    }

    @Test
    public void testMutate() {
        IntChromosome chromosome = (IntChromosome) strategy.getTemplateChromosome();
        for (int i = 0; i < 100; i++) {
            for (int j = 0; j < 100; j++) {
                strategy.mutate(chromosome);
            }
        }
        asertLengthAndUniqueness(chromosome);
    }

    @Test
    public void testSinglePointCrossover() {
        IntChromosome[] chromosomes = new IntChromosome[] {
            (IntChromosome) strategy.getTemplateChromosome(),
            (IntChromosome) strategy.getTemplateChromosome(),
            (IntChromosome) strategy.getTemplateChromosome() };
        for (int i = 0; i < 100; i++) {
            // do a bunch of crossovers among the three, choosing different
            // parents each time, and verify that each offspring has the right
            // length and is unique.
            int parent1 = i % 3;
            int parent2 = (i + 1) % 3;
            int offspring = (i + 2) % 3;
            chromosomes[offspring] = (IntChromosome) strategy.singlePointCrossover(
                chromosomes[parent1],
                chromosomes[parent2]);
            asertLengthAndUniqueness(chromosomes[offspring]);
        }
    }
}
