package main;

import ga.Algorithm;
import ga.problems.Problem;
import ga.problems.ProblemTSP;

import java.util.Random;

public class Exe {

    private static long startTime;
    private static Random random = new Random();

    public static void main(String args[]) {
        // defaults
        int allelesPerChromosome = 512;
        int testIterations = 100;
        int binaryTournamentRounds = 0;
        // int geneticIterations = 50000;
        int[] multipleGeneticIterations = new int[] { 1000, 3162, 10000, 31622, 100000, 316228 };
        double targetScore;
        Problem problem;
        int populationSizeMin = 2;
        int populationSizeMax = 50;
        double crossoverMin = 0.00;
        double crossoverMax = 1.00;
        double mutationsPerChromosomeMin = 0.00;
        double mutationsPerChromosomeMax = 3.00;

        // OneMax
        // targetScore = (double) allelesPerChromosome;

        // PPeaks
        // targetScore = 1.0;

        // TSP
        // String fileName = "Berlin52.txt";
        String fileName = "ch150.txt";
        targetScore = 0; // I don't want the TSP to halt

        for (int geneticIterations : multipleGeneticIterations) {
            // System.out.println("Iterations\tPopulationSize\tNumAlleles\tMutationProbability\tCrossoverProbability\tScore");
            for (int i = 0; i < testIterations; i++) {
                int populationSize = (int) (random.nextDouble() * (populationSizeMax - populationSizeMin))
                        + populationSizeMin;
                double crossoverProbability = (random.nextDouble() * (crossoverMax - crossoverMin))
                        + crossoverMin;
                double mutationsPerChromosome = (random.nextDouble() * (mutationsPerChromosomeMax - mutationsPerChromosomeMin))
                        + mutationsPerChromosomeMin;

                // problem = new ProblemOneMax(
                // allelesPerChromosome,
                // crossoverProbability,
                // mutationsPerChromosome);
                // problem = new ProblemPPeaks(
                // allelesPerChromosome,
                // crossoverProbability,
                // mutationsPerChromosome);
                problem = new ProblemTSP(crossoverProbability, mutationsPerChromosome, fileName);

                runProblem(
                    populationSize,
                    targetScore,
                    geneticIterations,
                    problem,
                    binaryTournamentRounds);
            }
        }
    }

    private static void runProblem(int populationSize, double targetScore, long geneticIterations,
            Problem problem, int binaryTournamentRounds) {
        // build the environment
        Algorithm geneticAlgorithm = new Algorithm(problem, populationSize);

        // run the algorithm
        System.out.printf("%d\t%d\t", geneticIterations, populationSize);
        problem.print();
        startTimer();
        for (long step = 0; step < geneticIterations; step++) {
            geneticAlgorithm.goOneStep(binaryTournamentRounds);
            if (problem.resultHasBeenFound(geneticAlgorithm, targetScore)) {
                // System.out.printf("Solution Found! After %d steps\n", step);
                break;
            }
        }
        // Print the solution
        stopTimer();
        System.out.printf("%f\n", geneticAlgorithm.getBestKnownIndividual().getScore());
    }

    private static void startTimer() {
        startTime = System.currentTimeMillis();
    }

    private static void stopTimer() {
        long elapsed = System.currentTimeMillis() - startTime;
        // System.out.printf("Algorithm finished in %d ms.\n", elapsed);
        System.out.printf("%d\t", elapsed);
    }
}
